<?php

namespace Beejees\Product\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Beejees\Product\Forms\Carrier as CarrierForm;
use Beejees\Product\Http\Requests\CarrierRequest;
use Beejees\Product\Interfaces\CarrierRepositoryInterface;
use Beejees\Product\Repositories\Eloquent\Filters\CarrierResourceFilter;
use Beejees\Product\Repositories\Eloquent\Presenters\CarrierListPresenter;

/**
 * Resource controller class for carrier.
 */
class CarrierResourceController extends BaseController
{

    /**
     * Initialize carrier resource controller.
     *
     *
     * @return null
     */
    public function __construct(CarrierRepositoryInterface $carrier)
    {
        parent::__construct();
        $this->form = CarrierForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('beejees.product.modules'), 'product', guard_url('product'));
        $this->repository = $carrier;
    }

    /**
     * Display a list of carrier.
     *
     * @return Response
     */
    public function index(CarrierRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(CarrierResourceFilter::class)
            ->setPresenter(CarrierListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('product::carrier.names'))
            ->view('product::carrier.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display carrier.
     *
     * @param Request $request
     * @param Model   $carrier
     *
     * @return Response
     */
    public function show(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('product::carrier.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('product::carrier.show')
            ->output();
    }

    /**
     * Show the form for creating a new carrier.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('product::carrier.name'))
            ->view('product::carrier.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new carrier.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('product::carrier.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('product/carrier/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/product/carrier'))
                ->redirect();
        }

    }

    /**
     * Show carrier for editing.
     *
     * @param Request $request
     * @param Model   $carrier
     *
     * @return Response
     */
    public function edit(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('product::carrier.name'))
            ->view('product::carrier.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the carrier.
     *
     * @param Request $request
     * @param Model   $carrier
     *
     * @return Response
     */
    public function update(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('product::carrier.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/carrier/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/carrier/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the carrier.
     *
     * @param Model   $carrier
     *
     * @return Response
     */
    public function destroy(CarrierRequest $request, CarrierRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('product::carrier.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('product/carrier/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('product/carrier/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
